@@ -14,16 +14,15 @@ from six import string_types
 
 from . import browserlib
 from . import _glyph_functions as gf
-from .deprecate import deprecated
 from .document import Document
 from .embed import notebook_div, file_html, autoload_server
 from .models import (
     Axis, FactorRange, Grid, GridPlot, HBox, Legend, LogAxis, Plot, Tool, VBox, Widget
 )
-from .palettes import brewer
 from .plotting_helpers import (
     get_default_color, get_default_alpha, _handle_1d_data_args, _list_attr_splat,
-    _get_range, _get_axis_class, _get_num_minor_ticks, _tool_from_string
+    _get_range, _get_axis_class, _get_num_minor_ticks, _tool_from_string,
+    _process_tools_arg
 )
 from .resources import Resources
 from .session import DEFAULT_SERVER_URL, Session
@@ -100,53 +99,8 @@ class Figure(Plot):
             elif y_axis_location == "right":
                 self.right.append(yaxis)
 
-        tool_objs = []
-        temp_tool_str = ""
-
-        if isinstance(tools, (list, tuple)):
-            for tool in tools:
-                if isinstance(tool, Tool):
-                    tool_objs.append(tool)
-                elif isinstance(tool, string_types):
-                    temp_tool_str += tool + ','
-                else:
-                    raise ValueError("tool should be a string or an instance of Tool class")
-            tools = temp_tool_str
-
-        # Remove pan/zoom tools in case of categorical axes
-        remove_pan_zoom = (isinstance(self.x_range, FactorRange) or
-                           isinstance(self.y_range, FactorRange))
-
-        repeated_tools = []
-        removed_tools = []
-
-        for tool in re.split(r"\s*,\s*", tools.strip()):
-            # re.split will return empty strings; ignore them.
-            if tool == "":
-                continue
-
-            if remove_pan_zoom and ("pan" in tool or "zoom" in tool):
-                removed_tools.append(tool)
-                continue
-
-            tool_obj = _tool_from_string(tool)
-            tool_obj.plot = self
-
-            tool_objs.append(tool_obj)
-
-        self.tools.extend(tool_objs)
-
-        for typename, group in itertools.groupby(sorted([ tool.__class__.__name__ for tool in self.tools ])):
-            if len(list(group)) > 1:
-                repeated_tools.append(typename)
-
-        if repeated_tools:
-            warnings.warn("%s are being repeated" % ",".join(repeated_tools))
-
-        if removed_tools:
-            warnings.warn("categorical plots do not support pan and zoom operations.\n"
-                          "Removing tool(s): %s" %', '.join(removed_tools))
-
+        tool_objs = _process_tools_arg(self, tools)
+        self.add_tools(*tool_objs)
 
     def _axis(self, *sides):
         objs = []
@@ -326,15 +280,6 @@ def curdoc():
     except (ImportError, RuntimeError, AttributeError):
         return _default_document
 
-@deprecated("Bokeh 0.7", "bokeh.plotting.Figure objects")
-def curplot():
-    ''' Return the current default plot object.
-
-    Returns:
-        plot : the current default plot (or None)
-    '''
-    return curdoc().curplot()
-
 def cursession():
     ''' Return the current session, if there is one.
 
@@ -362,20 +307,6 @@ def reset_output():
     _default_file = None
     _default_notebook = None
 
-@deprecated("Bokeh 0.7", "methods on bokeh.plotting.Figure objects")
-def hold(value=True):
-    ''' Set or clear the plot hold status on the current document.
-
-    This is a convenience function that acts on the current document, and is equivalent to curdoc().hold(...)
-
-    Args:
-        value (bool, optional) : whether hold should be turned on or off (default: True)
-
-    Returns:
-        None
-    '''
-    curdoc().hold(value)
-
 def figure(**kwargs):
     ''' Activate a new figure for plotting.
 
@@ -384,12 +315,22 @@ def figure(**kwargs):
     This function accepts all plot style keyword parameters.
 
     Returns:
-        None
+       figure : a new :class:`Plot <bokeh.models.plots.Plot>`
 
     '''
+    if 'plot_width' in kwargs and 'width' in kwargs:
+        raise ValueError("figure() called but both plot_width and width supplied, supply only one")
+    if 'plot_height' in kwargs and 'height' in kwargs:
+        raise ValueError("figure() called but both plot_height and height supplied, supply only one")
+    if 'height' in kwargs:
+        kwargs['plot_height'] = kwargs.pop('height')
+    if 'width' in kwargs:
+        kwargs['plot_width'] = kwargs.pop('width')
+
     fig = Figure(**kwargs)
     curdoc()._current_plot = fig
-    curdoc().add(fig)
+    if curdoc().autoadd:
+        curdoc().add(fig)
     return fig
 
 def output_server(docname, session=None, url="default", name=None, clear=True):
@@ -433,20 +374,18 @@ def output_server(docname, session=None, url="default", name=None, clear=True):
         if not _default_session:
             _default_session = Session(name=name, root_url=url)
         session = _default_session
+    else:
+        _default_session = session
     session.use_doc(docname)
     session.load_document(curdoc())
     if clear:
         curdoc().clear()
 
-def output_notebook(url=None, docname=None, session=None, name=None,
-                    force=False):
+def output_notebook(url=None, docname=None, session=None, name=None):
     if session or url or name:
         if docname is None:
             docname = "IPython Session at %s" % time.ctime()
         output_server(docname, url=url, session=session, name=name)
-    else:
-        from . import load_notebook
-        load_notebook(force=force)
     global _default_notebook
     _default_notebook = True
 
@@ -474,7 +413,7 @@ def output_file(filename, title="Bokeh Plot", autosave=False, mode="inline", roo
     global _default_file
     _default_file = {
         'filename'  : filename,
-        'resources' : Resources(mode=mode, root_dir=root_dir, minified=False),
+        'resources' : Resources(mode=mode, root_dir=root_dir),
         'autosave'  : autosave,
         'title'     : title,
     }
@@ -483,19 +422,20 @@ def output_file(filename, title="Bokeh Plot", autosave=False, mode="inline", roo
         print("Session output file '%s' already exists, will be overwritten." % filename)
 
 
-def show(obj=None, browser=None, new="tab", url=None):
-    """ 'shows' a plot object or the current plot, by auto-raising the window or tab
-    displaying the current plot (for file/server output modes) or displaying
-    it in an output cell (IPython notebook).
+def show(obj, browser=None, new="tab", url=None):
+    """ Immediately display a plot object.
+
+    In an IPython/Jupyter notebook, the output is displayed in an output cell.
+    Otherwise, a browser window or tab is autoraised to display the plot object.
 
     Args:
-        obj (Widget/Plot object, optional): it accepts a plot object and just shows it.
+        obj (Widget/Plot object): a plot object to display
 
         browser (str, optional) : browser to show with (default: None)
             For systems that support it, the **browser** argument allows specifying
             which browser to display in, e.g. "safari", "firefox", "opera",
-            "windows-default".  (See the webbrowser module documentation in the
-            standard lib for more details.)
+            "windows-default" (see the webbrowser module documentation in the
+            standard lib for more details).
 
         new (str, optional) : new file output mode (default: "tab")
             For file-based output, opens or raises the browser window
@@ -510,23 +450,14 @@ def show(obj=None, browser=None, new="tab", url=None):
     new_param = {'tab': 2, 'window': 1}[new]
 
     controller = browserlib.get_browser_controller(browser=browser)
-    if obj is None:
-        if notebook:
-            plot = curplot()
-        else:
-            plot = curdoc()
-    else:
-        plot = obj
-    if not plot:
-        warnings.warn("No current plot to show. Use renderer functions (circle, rect, etc.) to create a current plot (see http://bokeh.pydata.org/index.html)")
-        return
+
     if notebook and session:
         push(session=session)
-        snippet = autoload_server(plot, cursession())
+        snippet = autoload_server(obj, cursession())
         publish_display_data({'text/html': snippet})
 
     elif notebook:
-        publish_display_data({'text/html': notebook_div(plot)})
+        publish_display_data({'text/html': notebook_div(obj)})
 
     elif session:
         push()
@@ -536,11 +467,11 @@ def show(obj=None, browser=None, new="tab", url=None):
             controller.open(session.object_link(curdoc().context))
 
     elif filename:
-        save(filename, obj=plot)
+        save(filename, obj=obj)
         controller.open("file://" + os.path.abspath(filename), new=new_param)
 
 
-def save(filename=None, resources=None, obj=None):
+def save(filename=None, resources=None, obj=None, title=None):
     """ Updates the file with the data for the current document.
 
     If a filename is supplied, or output_file(...) has been called, this will
@@ -550,11 +481,13 @@ def save(filename=None, resources=None, obj=None):
         filename (str, optional) : filename to save document under (default: None)
             if `filename` is None, the current output_file(...) filename is used if present
         resources (Resources, optional) : BokehJS resource config to use
-            if `resources` is None, the current default resource config is used
+            if `resources` is None, the current default resource config is used, failing that resources.INLINE is used
 
         obj (Document or Widget/Plot object, optional)
             if provided, then this is the object to save instead of curdoc()
             and its curplot()
+        title (str, optional) : title of the bokeh plot (default: None)
+        	if 'title' is None, the current default title config is used, failing that 'Bokeh Plot' is used
 
     Returns:
         None
@@ -566,13 +499,22 @@ def save(filename=None, resources=None, obj=None):
     if resources is None and _default_file:
         resources = _default_file['resources']
 
+    if title is None and _default_file:
+        title = _default_file['title']
+
     if not filename:
         warnings.warn("save() called but no filename was supplied and output_file(...) was never called, nothing saved")
         return
 
     if not resources:
-        warnings.warn("save() called but no resources was supplied and output_file(...) was never called, nothing saved")
-        return
+        warnings.warn("save() called but no resources was supplied and output_file(...) was never called, defaulting to resources.INLINE")
+        from .resources import INLINE
+        resources = INLINE
+
+
+    if not title:
+        warnings.warn("save() called but no title was supplied and output_file(...) was never called, using default title 'Bokeh Plot'")
+        title = "Bokeh Plot"
 
     if obj is None:
         if not curplot():
@@ -587,7 +529,7 @@ def save(filename=None, resources=None, obj=None):
     else:
         raise RuntimeError("Unable to save object of type '%s'" % type(obj))
 
-    html = file_html(doc, resources, _default_file['title'])
+    html = file_html(doc, resources, title)
     with io.open(filename, "w", encoding="utf-8") as f:
         f.write(decode_utf8(html))
 
@@ -615,170 +557,26 @@ def push(session=None, document=None):
     else:
         warnings.warn("push() called but no session was supplied and output_server(...) was never called, nothing pushed")
 
-def _doc_wrap(func):
-    extra_doc = "\nThis is a convenience function that acts on the current plot of the current document, and is equivalent to curlot().%s(...)\n\n" % func.__name__
-    func.__doc__ = getattr(gf, func.__name__).__doc__ + extra_doc
-    return deprecated(
-        "Bokeh 0.7",
-        "glyph methods on plots, e.g. plt.%s(...)" % func.__name__
-    )(func)
-
-def _plot_function(__func__, *args, **kwargs):
-    retval = __func__(curdoc(), *args, **kwargs)
-    if cursession() and curdoc().autostore:
-        push()
-    if _default_file and _default_file['autosave']:
-        save()
-    return retval
-
-@_doc_wrap
-def annular_wedge(x, y, inner_radius, outer_radius, start_angle, end_angle, **kwargs):
-    return _plot_function(gf.annular_wedge, x, y, inner_radius, outer_radius, start_angle, end_angle, **kwargs)
-
-@_doc_wrap
-def annulus(x, y, inner_radius, outer_radius, **kwargs):
-    return _plot_function(gf.annulus, x, y, inner_radius, outer_radius, **kwargs)
-
-@_doc_wrap
-def arc(x, y, radius, start_angle, end_angle, **kwargs):
-    return _plot_function(gf.arc, x, y, radius, start_angle, end_angle, **kwargs)
-
-@_doc_wrap
-def asterisk(x, y, **kwargs):
-    return _plot_function(gf.asterisk, x, y, **kwargs)
-
-@_doc_wrap
-def bezier(x0, y0, x1, y1, cx0, cy0, cx1, cy1, **kwargs):
-    return _plot_function(gf.bezier, x0, y0, x1, y1, cx0, cy0, cx1, cy1, **kwargs)
-
-@_doc_wrap
-def circle(x, y, **kwargs):
-    return _plot_function(gf.circle, x, y, **kwargs)
-
-@_doc_wrap
-def circle_cross(x, y, **kwargs):
-    return _plot_function(gf.circle_cross, x, y, **kwargs)
-
-@_doc_wrap
-def circle_x(x, y, **kwargs):
-    return _plot_function(gf.circle_x, x, y, **kwargs)
-
-@_doc_wrap
-def cross(x, y, **kwargs):
-    return _plot_function(gf.cross, x, y, **kwargs)
-
-@_doc_wrap
-def diamond(x, y, **kwargs):
-    return _plot_function(gf.diamond, x, y, **kwargs)
-
-@_doc_wrap
-def diamond_cross(x, y, **kwargs):
-    return _plot_function(gf.diamond_cross, x, y, **kwargs)
-
-@_doc_wrap
-def image(image, x, y, dw, dh, **kwargs):
-    return _plot_function(gf.image, image, x, y, dw, dh, **kwargs)
-
-@_doc_wrap
-def image_rgba(image, x, y, dw, dh, **kwargs):
-    return _plot_function(gf.image_rgba, image, x, y, dw, dh, **kwargs)
-
-@_doc_wrap
-def image_url(url, x, y, angle, **kwargs):
-    return _plot_function(gf.image_url, url, x, y, angle, **kwargs)
-
-@_doc_wrap
-def inverted_triangle(x, y, **kwargs):
-    return _plot_function(gf.inverted_triangle, x, y, **kwargs)
-
-@_doc_wrap
-def line(x, y, **kwargs):
-    return _plot_function(gf.line, x, y, **kwargs)
-
-@_doc_wrap
-def multi_line(xs, ys, **kwargs):
-    return _plot_function(gf.multi_line, xs, ys, **kwargs)
-
-@_doc_wrap
-def oval(x, y, width, height, **kwargs):
-    return _plot_function(gf.oval, x, y, width, height, **kwargs)
-
-@_doc_wrap
-def patch(x, y, **kwargs):
-    return _plot_function(gf.patch, x, y, **kwargs)
-
-@_doc_wrap
-def patches(xs, ys, **kwargs):
-    return _plot_function(gf.patches, xs, ys, **kwargs)
-
-@_doc_wrap
-def quad(left, right, top, bottom, **kwargs):
-    return _plot_function(gf.quad, left, right, top, bottom, **kwargs)
-
-@_doc_wrap
-def quadratic(x0, y0, x1, y1, cx, cy, **kwargs):
-    return _plot_function(gf.quadratic, x0, y0, x1, y1, cx, cy, **kwargs)
-
-@_doc_wrap
-def ray(x, y, length, angle, **kwargs):
-    return _plot_function(gf.ray, x, y, length, angle, **kwargs)
-
-@_doc_wrap
-def rect(x, y, width, height, **kwargs):
-    return _plot_function(gf.rect, x, y, width, height, **kwargs)
-
-@_doc_wrap
-def segment(x0, y0, x1, y1, **kwargs):
-    return _plot_function(gf.segment, x0, y0, x1, y1, **kwargs)
-
-@_doc_wrap
-def square(x, y, **kwargs):
-    return _plot_function(gf.square, x, y, **kwargs)
-
-@_doc_wrap
-def square_cross(x, y, **kwargs):
-    return _plot_function(gf.square_cross, x, y, **kwargs)
-
-@_doc_wrap
-def square_x(x, y, **kwargs):
-    return _plot_function(gf.square_x, x, y, **kwargs)
-
-@_doc_wrap
-def text(x, y, text, angle, **kwargs):
-    return _plot_function(gf.text, x, y, text, angle, **kwargs)
-
-@_doc_wrap
-def triangle(x, y, **kwargs):
-    return _plot_function(gf.triangle, x, y, **kwargs)
-
-@_doc_wrap
-def wedge(x, y, radius, start_angle, end_angle, **kwargs):
-    return _plot_function(gf.wedge, x, y, radius, start_angle, end_angle, **kwargs)
-
-@_doc_wrap
-def x(x, y, **kwargs):
-    return _plot_function(gf.x, x, y, **kwargs)
-
-_marker_types = {
-    "asterisk": asterisk,
-    "circle": circle,
-    "circle_cross": circle_cross,
-    "circle_x": circle_x,
-    "cross": cross,
-    "diamond": diamond,
-    "diamond_cross": diamond_cross,
-    "inverted_triangle": inverted_triangle,
-    "square": square,
-    "square_x": square_x,
-    "square_cross": square_cross,
-    "triangle": triangle,
-    "x": x,
-    "*": asterisk,
-    "+": cross,
-    "o": circle,
-    "ox": circle_x,
-    "o+": circle_cross,
-}
+_marker_types = [
+    "asterisk",
+    "circle",
+    "circle_cross",
+    "circle_x",
+    "cross",
+    "diamond",
+    "diamond_cross",
+    "inverted_triangle",
+    "square",
+    "square_x",
+    "square_cross",
+    "triangle",
+    "x",
+    "*",
+    "+",
+    "o",
+    "ox",
+    "o+",
+]
 
 def markers():
     """ Prints a list of valid marker types for scatter()
@@ -788,165 +586,39 @@ def markers():
     """
     print(list(sorted(_marker_types.keys())))
 
-
 _color_fields = set(["color", "fill_color", "line_color"])
 _alpha_fields = set(["alpha", "fill_alpha", "line_alpha"])
 
-@deprecated("Bokeh 0.7", "bokeh.plotting.Figure.scatter")
-def scatter(*args, **kwargs):
-    """ Creates a scatter plot of the given x and y items.
+def _deduplicate_plots(plot, subplots):
+    doc = curdoc()
+    doc.context.children = list(set(doc.context.children) - set(subplots))
+    doc.add(plot)
+    doc._current_plot = plot # TODO (bev) don't use private attrs
 
-    Args:
-        *args : The data to plot.  Can be of several forms:
+def _push_or_save():
+    if cursession() and curdoc().autostore:
+        push()
+    if _default_file and _default_file['autosave']:
+        save()
 
-            (X, Y)
-                Two 1D arrays or iterables
-            (XNAME, YNAME)
-                Two bokeh DataSource/ColumnsRef
-
-        marker (str, optional): a valid marker_type, defaults to "circle"
-        color (color value, optional): shorthand to set both fill and line color
-
-    All the :ref:`userguide_objects_line_properties` and :ref:`userguide_objects_fill_properties` are
-    also accepted as keyword parameters.
-
-    Examples:
-
-        >>> scatter([1,2,3],[4,5,6], fill_color="red")
-        >>> scatter("data1", "data2", source=data_source, ...)
-
-    """
-    ds = kwargs.get("source", None)
-    names, datasource = _handle_1d_data_args(args, datasource=ds)
-    kwargs["source"] = datasource
-
-    markertype = kwargs.get("marker", "circle")
-
-    # TODO: How to handle this? Just call curplot()?
-    if not len(_color_fields.intersection(set(kwargs.keys()))):
-        kwargs['color'] = get_default_color()
-    if not len(_alpha_fields.intersection(set(kwargs.keys()))):
-        kwargs['alpha'] = get_default_alpha()
-
-    if markertype not in _marker_types:
-        raise ValueError("Invalid marker type '%s'. Use markers() to see a list of valid marker types." % markertype)
-    return _marker_types[markertype](*args, **kwargs)
-
-def gridplot(plot_arrangement, name=None, **kwargs):
+def gridplot(plot_arrangement, **kwargs):
     """ Generate a plot that arranges several subplots into a grid.
 
     Args:
-        plot_arrangement (list[:class:`Plot <bokeh.models.Plot>`]) : plots to arrange in a grid
-        name (str) : name for this plot
+        plot_arrangement (nested list of Plots) : plots to arrange in a grid
         **kwargs: additional attributes to pass in to GridPlot() constructor
 
     .. note:: `plot_arrangement` can be nested, e.g [[p1, p2], [p3, p4]]
 
     Returns:
-        grid_plot: the current :class:`GridPlot <bokeh.models.GridPlot>`
+        grid_plot: a new :class:`GridPlot <bokeh.models.plots.GridPlot>`
     """
     grid = GridPlot(children=plot_arrangement, **kwargs)
-    if name:
-        grid._id = name
-    # Walk the plot_arrangement and remove them from the plotcontext,
-    # so they don't show up twice
     subplots = itertools.chain.from_iterable(plot_arrangement)
-    curdoc().context.children = list(set(curdoc().context.children) - set(subplots))
-    curdoc().add(grid)
-    curdoc()._current_plot = grid # TODO (bev) don't use private attrs
-
-    if _default_session:
-        push()
-    if _default_file and _default_file['autosave']:
-        save()
+    _deduplicate_plots(grid, subplots)
+    _push_or_save()
     return grid
 
-# TODO (bev) remove after 0.7
-def _axis(*sides):
-    p = curplot()
-    if p is None:
-        return None
-    objs = []
-    for s in sides:
-        objs.extend(getattr(p, s, []))
-    axis = [obj for obj in objs if isinstance(obj, Axis)]
-    return _list_attr_splat(axis)
-
-@deprecated("Bokeh 0.7", "bokeh.plotting.Figure.xaxis")
-def xaxis():
-    """ Get the current `x` axis object(s)
-
-    Returns:
-        Returns x-axis object or splattable list of x-axis objects on the current plot
-    """
-    return _axis("above", "below")
-
-@deprecated("Bokeh 0.7", "bokeh.plotting.Figure.yaxis")
-def yaxis():
-    """ Get the current `y` axis object(s)
-
-    Returns:
-        Returns y-axis object or splattable list of y-axis objects on the current plot
-    """
-    return _axis("left", "right")
-
-@deprecated("Bokeh 0.7", "bokeh.plotting.Figure.axis")
-def axis():
-    """ Get all the current axis objects
-
-    Returns:
-        Returns axis object or splattable list of axis objects on the current plot
-    """
-    return _list_attr_splat(xaxis() + yaxis())
-
-@deprecated("Bokeh 0.7", "bokeh.plotting.Figure.legend")
-def legend():
-    """ Get the current :class:`legend <bokeh.models.Legend>` object(s)
-
-    Returns:
-        Returns legend object or splattable list of legend objects on the current plot
-    """
-    p = curplot()
-    if p is None:
-        return None
-    legends = [obj for obj in p.renderers if isinstance(obj, Legend)]
-    return _list_attr_splat(legends)
-
-# TODO (bev): remove after 0.7
-def _grid(dimension):
-    p = curplot()
-    if p is None:
-        return None
-    grid = [obj for obj in p.renderers if isinstance(obj, Grid) and obj.dimension==dimension]
-    return _list_attr_splat(grid)
-
-@deprecated("Bokeh 0.7", "bokeh.plotting.Figure.xgrid")
-def xgrid():
-    """ Get the current `x` :class:`grid <bokeh.models.Grid>` object(s)
-
-    Returns:
-        Returns legend object or splattable list of legend objects on the current plot
-    """
-    return _grid(0)
-
-@deprecated("Bokeh 0.7", "bokeh.plotting.Figure.ygrid")
-def ygrid():
-    """ Get the current `y` :class:`grid <bokeh.models.Grid>` object(s)
-
-    Returns:
-        Returns y-grid object or splattable list of y-grid objects on the current plot
-    """
-    return _grid(1)
-
-@deprecated("Bokeh 0.7", "bokeh.plotting.Figure.grid")
-def grid():
-    """ Get the current :class:`grid <bokeh.models.Grid>` object(s)
-
-    Returns:
-        Returns grid object or splattable list of grid objects on the current plot
-    """
-    return _list_attr_splat(xgrid() + ygrid())
-
 def load_object(obj):
     """updates object from the server
     """
